// Fill out your copyright notice in the Description page of Project Settings.

#include "ViveportSDKPrivatePCH.h"
#include "ViveportUserStats.h"
#include <string>

#include "viveport_api.h"

static ViveportApiStatus* is_ready_status;
static ViveportApiStatus* download_stats_status;
static ViveportApiStatus* upload_stats_status;
static ViveportApiStatus* upload_leaderboard_status;
static ViveportApiStatus* download_leaderboard_status;

void IsReadyHandler(int);
void DownloadStatsHandler(int);
void UploadStatsHandler(int);
void DownloadLeaderboardScoreHandler(int);
void UploadLeaderboardScoreHandler(int);

bool UViveportUserStats::IsReady(ViveportApiStatus *callback)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        is_ready_status = callback;
        ret = ViveportUserStats()->IsReady(IsReadyHandler);
    }
    return ret == 1;
}

int UViveportUserStats::DownloadStats(ViveportApiStatus *callback)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        download_stats_status = callback;
        ret = ViveportUserStats()->DownloadStats(DownloadStatsHandler);
    }
    return ret;
}

int UViveportUserStats::UploadStats(ViveportApiStatus *callback)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        upload_stats_status = callback;
        ret = ViveportUserStats()->UploadStats(UploadStatsHandler);
    }
    return ret;
}

int UViveportUserStats::GetStat(const FString fName, int default_value)
{
    int data = default_value;
    if (FViveportSDKModule::IsAvailable())
    {
        ViveportUserStats()->GetStat(TCHAR_TO_ANSI(*fName), &data);
    }
    return data;
}

float UViveportUserStats::GetStat(const FString fName, float default_value)
{
    float data = default_value;
    if (FViveportSDKModule::IsAvailable())
    {
        ViveportUserStats()->GetStat(TCHAR_TO_ANSI(*fName), &data);
    }
    return data;
}

int UViveportUserStats::SetStat(const FString fName, int data)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ret = ViveportUserStats()->SetStat(TCHAR_TO_ANSI(*fName), data);
    }
    return ret;
}

int UViveportUserStats::SetStat(const FString fName, float fData)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ret = ViveportUserStats()->SetStat(TCHAR_TO_ANSI(*fName), fData);
    }
    return ret;
}

bool UViveportUserStats::GetAchievement(const FString fName)
{
    int achieved = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ViveportUserStats()->GetAchievement(TCHAR_TO_ANSI(*fName), &achieved);
    }
    return achieved == 1;
}

int UViveportUserStats::GetAchievementUnlockTime(const FString fName)
{
    int unlockTime = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ViveportUserStats()->GetAchievementUnlockTime(TCHAR_TO_ANSI(*fName), &unlockTime);
    }
    return unlockTime;
}

int UViveportUserStats::SetAchievement(const FString fName)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ret = ViveportUserStats()->SetAchievement(TCHAR_TO_ANSI(*fName));
    }
    return ret;
}

int UViveportUserStats::ClearAchievement(const FString fName)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        ret = ViveportUserStats()->ClearAchievement(TCHAR_TO_ANSI(*fName));
    }
    return ret;
}

int UViveportUserStats::DownloadLeaderboardScores(ViveportApiStatus *callback,
    const FString fLeaderboardName,
    UELeaderboardDataDownload eLeaderboardDataDownload,
    UELeaderboardDataTimeRange eLeaderboardDataTimeRange,
    int nRangeStart,
    int nRangeEnd)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        download_leaderboard_status = callback;

        ret = ViveportUserStats()->DownloadLeaderboardScores(DownloadLeaderboardScoreHandler,
                                                            TCHAR_TO_ANSI(*fLeaderboardName),
                                                            (ELeaderboardDataDownload) eLeaderboardDataDownload,
                                                            (ELeaderboardDataTimeRange) eLeaderboardDataTimeRange,
                                                            nRangeStart,
                                                            nRangeEnd);
    }
    return ret;
}

int UViveportUserStats::UploadLeaderboardScore(ViveportApiStatus  *callback, const FString fName, int nScore)
{
    int ret = 0;
    if (FViveportSDKModule::IsAvailable())
    {
        upload_leaderboard_status = callback;
        ret = ViveportUserStats()->UploadLeaderboardScore(UploadLeaderboardScoreHandler,
                                                        TCHAR_TO_ANSI(*fName),
                                                        nScore);
    }
    return ret;
}

ULeaderboardScore UViveportUserStats::GetLeaderboardScore(int index)
{
    ULeaderboardScore uLeaderboardScore;
    if (FViveportSDKModule::IsAvailable())
    {
        LeaderboardScore leaderboardScore;
        ViveportUserStats()->GetLeaderboardScore(index, &leaderboardScore);

        uLeaderboardScore.Rank = leaderboardScore.Rank;
        uLeaderboardScore.Score = leaderboardScore.Score;
        uLeaderboardScore.UserName = UTF8_TO_TCHAR(leaderboardScore.UserName);
    }
    return uLeaderboardScore;
}

int UViveportUserStats::GetLeaderboardScoreCount()
{
    if (FViveportSDKModule::IsAvailable())
    {
        return ViveportUserStats()->GetLeaderboardScoreCount();
    }
    return 0;
}

UELeaderboardSortMethod UViveportUserStats::GetLeaderboardSortMethod()
{
    if (FViveportSDKModule::IsAvailable())
    {
        return (UELeaderboardSortMethod)ViveportUserStats()->GetLeaderboardSortMethod();
    }
    return u_ELeaderboardSortMethodNone;
}

UELeaderboardDisplayType UViveportUserStats::GetLeaderboardDisplayType()
{
    if (FViveportSDKModule::IsAvailable())
    {
        return (UELeaderboardDisplayType)ViveportUserStats()->GetLeaderboardDisplayType();
    }
    return u_ELeaderboardDisplayTypeNumeric;
}

void IsReadyHandler(int nResult)
{
    return nResult == 0 ? is_ready_status->OnSuccess() : is_ready_status->OnFailure(nResult);
}

void DownloadStatsHandler(int nResult)
{
    return nResult == 0 ? download_stats_status->OnSuccess() : download_stats_status->OnFailure(nResult);
}

void UploadStatsHandler(int nResult)
{
    return nResult == 0 ? upload_stats_status->OnSuccess() : upload_stats_status->OnFailure(nResult);
}

void DownloadLeaderboardScoreHandler(int nResult)
{
    return nResult == 0 ? download_leaderboard_status->OnSuccess() : download_leaderboard_status->OnFailure(nResult);
}

void UploadLeaderboardScoreHandler(int nResult)
{
    return nResult == 0 ? upload_leaderboard_status->OnSuccess() : upload_leaderboard_status->OnFailure(nResult);
}